Package: NeuroDecodeR
Title: Decode Information from Neural Activity
Version: 0.2.0
Authors@R: c(
  person("Ethan", "Meyers", email = "ethan.meyers@gmail.com", 
  role = c("aut", "cre"))
  )
Description: Neural decoding is method of analyzing neural data that  
    uses a pattern classifiers to predict experimental conditions based 
    on neural activity. 'NeuroDecodeR' is a system of objects that 
    makes it easy to run neural decoding analyses. For more information
    on neural decoding see Meyers & Kreiman (2011)
    <doi:10.7551/mitpress/8404.003.0024>.
URL: https://emeyers.github.io/NeuroDecodeR/,
        https://github.com/emeyers/NeuroDecodeR
BugReports: https://github.com/emeyers/NeuroDecodeR/issues
Depends: R (>= 4.1.0)
License: GPL-3
RoxygenNote: 7.2.3
Imports: dplyr, doSNOW, e1071, forcats, foreach, ggplot2, gridExtra,
        magrittr, methods, purrr, R.matlab, scales, stats, stringr,
        tibble, tictoc, tidyr, utils
Suggests: knitr, rmarkdown, testthat
Encoding: UTF-8
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-03-15 04:47:30 UTC; emeyers
Author: Ethan Meyers [aut, cre]
Maintainer: Ethan Meyers <ethan.meyers@gmail.com>
Repository: CRAN
Date/Publication: 2024-03-15 11:40:02 UTC
Built: R 4.6.0; ; 2026-04-29 22:53:15 UTC; unix
