# Generated by roxygen2: do not edit by hand

S3method(aggregate_CV_split_results,rm_confusion_matrix)
S3method(aggregate_CV_split_results,rm_main_results)
S3method(aggregate_resample_run_results,rm_confusion_matrix)
S3method(aggregate_resample_run_results,rm_main_results)
S3method(get_data,ds_basic)
S3method(get_data,ds_generalization)
S3method(get_parameters,cl_max_correlation)
S3method(get_parameters,cl_poisson_naive_bayes)
S3method(get_parameters,cl_svm)
S3method(get_parameters,cv_standard)
S3method(get_parameters,ds_basic)
S3method(get_parameters,ds_generalization)
S3method(get_parameters,fp_select_k_features)
S3method(get_parameters,fp_zscore)
S3method(get_parameters,rm_confusion_matrix)
S3method(get_parameters,rm_main_results)
S3method(get_predictions,cl_max_correlation)
S3method(get_predictions,cl_poisson_naive_bayes)
S3method(get_predictions,cl_svm)
S3method(plot,label_repetition)
S3method(plot,raster_data)
S3method(plot,rm_confusion_matrix)
S3method(plot,rm_main_results)
S3method(preprocess_data,fp_select_k_features)
S3method(preprocess_data,fp_zscore)
S3method(run_decoding,cv_standard)
export(aggregate_CV_split_results)
export(aggregate_resample_run_results)
export(cl_max_correlation)
export(cl_poisson_naive_bayes)
export(cl_svm)
export(convert_matlab_raster_data)
export(create_binned_data)
export(cv_standard)
export(ds_basic)
export(ds_generalization)
export(fp_select_k_features)
export(fp_zscore)
export(get_data)
export(get_num_label_repetitions)
export(get_parameters)
export(get_predictions)
export(get_siteIDs_with_k_label_repetitions)
export(log_check_results_already_exist)
export(log_load_results_from_params)
export(log_load_results_from_result_name)
export(log_save_results)
export(plot_main_results)
export(preprocess_data)
export(read_matlab_raster_data)
export(read_raster_data)
export(rm_confusion_matrix)
export(rm_main_results)
export(run_decoding)
export(test_valid_ndr_object)
export(test_valid_raster_format)
import(dplyr)
import(foreach)
import(ggplot2)
import(magrittr)
importFrom(methods,is)
importFrom(stats,cor)
importFrom(stats,na.omit)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
